/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#pragma once

#include <array>

template <typename T, size_t MAX_SIZE>
class ArrayWithSize : public std::array<T, MAX_SIZE> {
	size_t num{0};

public:
	size_t size() const { return num; }
	bool empty() const { return size() == 0; }

	void push_back(const T &elem) {
		if (num < MAX_SIZE)
			(*this)[num++] = elem;
	}
	void clear() { num = 0; }
	const T *begin() const { return &(*this)[0]; }
	const T *end() const { return &(*this)[num]; }
};
