/*
 * Copyright (c) 2016-2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#include "cynara.hpp"
#include <stdexcept>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

using namespace ldp_cynara;

CynaraResult Cynara::check(const char* label, const std::string &privilege, uid_t uid) {
	(void)label;
	if (privilege.empty()) {
		return CynaraResult::ALLOW;
	}
	if (strcmp(label, "System::Privileged") == 0) {
		return CynaraResult::ALLOW;
	}
	if (uid == 9999) {
		if (privilege == "http://tizen.org/privilege/packagemanager.admin") {
			return CynaraResult::ALLOW;
		}
		return CynaraResult::DENY;
	}
	if (privilege == "privilege1") {
		if (uid == 9991 || uid == 9993) {
			return CynaraResult::ALLOW;
		}
		return CynaraResult::DENY;
	}
	if (privilege == "privilege2") {
		if (uid == 9992 || uid == 9993) {
			return CynaraResult::ALLOW;
		}
		return CynaraResult::DENY;
	}
	return CynaraResult::ALLOW;
}
