/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#pragma once

#include "policy.hpp"
#include "own_tree.hpp"
#include <vector>

namespace ldp_xml_parser {
/****************** PolicyBase: a base for policies ************************/
template <typename TI>
class PolicyBase {
protected:
	/** Vector with policy items */
	std::vector<TI> m_items;

public:
	typedef TI item_type;
	/** Adds given item to policy.
	 * \param[in] item Item to add to policy
	 */
	void addItem(TI &item) { m_items.push_back(std::move(item)); }

	const std::vector<TI> &getItems() const { return m_items; }
};

using PolicySend = PolicyBase<ItemSend>;
using PolicyReceive = PolicyBase<ItemReceive>;
using PolicyAccess = PolicyBase<ItemAccess>;

/****************** PolicyOwn ************************/
/** Class containing policy with ownership rules */
/* PolicyOwn is different than other policies, so it has its own class */
class PolicyOwn {
private:
	class OwnershipTree ownership_tree;

public:
	typedef ItemOwn item_type;

	/** Adds given item to tree by retrieving its name, decision and checking is it prefix.
	 * \param[in] item Item to add to policy
	 */
	void addItem(const ItemOwn &item) { ownership_tree.addItem(item); }

	const OwnershipTree &getTree() const { return ownership_tree; };
};

}
