#include "internal/include/fb_generated.h"
#include "internal/naive_policy_checker.hpp"
#include "internal/policy.hpp"
#include "internal/serializer.hpp"
#include "internal/storage_backend_serialized.hpp"
#include "internal/storage_backend_xml.hpp"
#include "internal/tslog.hpp"
#include <map>

using namespace ldp_xml_parser;
using namespace ldp_serialized;

struct OwnershipTest {
	Decision expected_result;
	uid_t user;
	gid_t group;
	const char* label;
	const char* service;
};

std::map<Decision, const char*> DECISIONS {
	{ Decision::ANY,   "ANY"   },
	{ Decision::ALLOW, "ALLOW" },
	{ Decision::DENY,  "DENY"  },
	{ Decision::CHECK, "CHECK" }
};

const int ROOT = 0;
const int GUEST = 9999;
const int GUEST1 = 9991;
const int GUEST2 = 9992;
const int GUEST12 = 9993;

/**
 * This test set tests ability to parse xml db
 * and check ownership privilege in many use cases
 * including prefix feature
 */
struct OwnershipTest ownership_tests[]={
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.ldpo.a"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.ldpoga"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.ldpogd"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.ldposa"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.ldpo.any_suffix" },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.ldpnotexistent"  },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.ldponotexistent" },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a"               },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.z"             },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.zz"            },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.aa"              },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.aaa"             },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a1"              },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a1.b"            },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a1.b1"           },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a1.c"            },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.b"               },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.b.c"             },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.b.z"             },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.c"               },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.c.c"             },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.c.z"             },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b"             },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.z"           },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.zz"          },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.bsth"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.bsthelse"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c"           },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.z.z"       },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d"         },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.z"       },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.e"       },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.esth"    },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.ee"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.e.z"     },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.e.z.z"   },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c.d.e.f"     },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1"          },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.z.z"      },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d"        },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.z"      },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.e"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.esth"   },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.ee"     },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.e.z"    },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.e.z.z"  },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.e.f"    },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c1.d.e.f.g"  },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.d"        },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.d.z"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.dd"       },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.d.e"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.d.e.f"    },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.d.e.f.z"  },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c2.d.e.fsth" },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3"          },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.z.z"      },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d"        },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.z"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.e"      },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.esth"   },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.ee"     },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.e.z"    },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.e.z.z"  },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.e.f"    },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.b.c3.d.e.f.g"  },
	{Decision::DENY,  ROOT,    ROOT,    "User::Shell", "org.tizen.a.c"             },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.c.z"           },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.d"             },
	{Decision::ALLOW, ROOT,    ROOT,    "User::Shell", "org.tizen.a.d.z"           },
	{Decision::CHECK, ROOT,    ROOT,    "User::Shell", "org.tizen.pok"             },
	{Decision::CHECK, ROOT,    ROOT,    "User::Shell", "org.tizen.pnope"           },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok"             },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pnope"           },
	{Decision::ALLOW, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1"            },
	{Decision::ALLOW, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.z"          },
	{Decision::DENY,  GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a"          },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b1"       },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b1.z"     },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b2"       },
	{Decision::DENY,  GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b2.z"     },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b3"       },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b3.z"     },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b4"       },
	{Decision::DENY,  GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a.b4.z"     },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a1"         },
	{Decision::CHECK, GUEST,   GUEST,   "User::Shell", "org.tizen.pok1.a1.z"       },
	{Decision::CHECK, GUEST1,  GUEST1,  "User::Shell", "org.tizen.pok2"            },
	{Decision::CHECK, GUEST2,  GUEST2,  "User::Shell", "org.tizen.pok2"            },
	{Decision::CHECK, GUEST12, GUEST12, "User::Shell", "org.tizen.pok2"            },
	{Decision::DENY,  GUEST1,  GUEST1,  "User::Shell", "org.tizen.pok2.a"          },
	{Decision::DENY,  GUEST2,  GUEST2,  "User::Shell", "org.tizen.pok2.a"          },
	{Decision::DENY,  GUEST12, GUEST12, "User::Shell", "org.tizen.pok2.a"          },
	{Decision::CHECK, GUEST1,  GUEST1,  "User::Shell", "org.tizen.pok2.a.b"        },
	{Decision::CHECK, GUEST2,  GUEST2,  "User::Shell", "org.tizen.pok2.a.b"        },
	{Decision::CHECK, GUEST12, GUEST12, "User::Shell", "org.tizen.pok2.a.b"        }
};

void ownershipTest_print(const struct OwnershipTest* t, Decision result) {
	printf("uid = %lu, gid = %lu, label = %s, service = %s, expected = %s, result = %s",
		   (unsigned long)t->user, (unsigned long)t->group, t->label, t->service, DECISIONS[t->expected_result], DECISIONS[result]);
}

template <typename DB>
bool ownership_test(const DB &db) {
	unsigned  i = 0;
	bool flag = true;

	for (auto const &test : ownership_tests) {
		auto m_item = MatchItemOwn(test.service);

		auto ret = db.getDecisionItemContextMandatory(m_item);

		if (ret.getDecision() == Decision::ANY)
			ret = db.getDecisionItemUser(test.user, m_item);

		if (ret.getDecision() == Decision::ANY)
			ret = db.getDecisionItemGroup(test.group, m_item);

		if (ret.getDecision() == Decision::ANY)
			ret = db.getDecisionItemContextDefault(m_item);

		auto decision = ret.getDecision();
		if (decision != test.expected_result) {
			printf("[ERROR][%d] ownership test failed: %s %s ", i, DECISIONS[test.expected_result], DECISIONS[decision]);
			ownershipTest_print(&test, decision);
			printf("\n");
			flag = false;
		}
	}
	return flag;
}

bool run_policy_db() {
	policy_checker_system().initDb("tests/default_deny/system.conf");
	auto &db = policy_checker_system().getPolicyDb();
	printf("POLICY_DB:\n");
	return ownership_test(db);
}

bool run_fb() {
	Serializer serializer;
	size_t size;
	uint8_t *buff = serializer.serialize("tests/default_deny/system.conf", size);

	const FB::File *file = FB::GetFile(buff);
	StorageBackendSerialized storage;
	storage.init(file);

	printf("FRAMEBUFFERS:\n");
	return ownership_test(storage);
}

bool run_xml() {
	return true;
}

bool run_tests() {
	return run_policy_db() && run_fb() && run_xml();
}

int main() {
	tslog::init();
	if (!run_tests())
		return -1;
return 0;
}
